//
// Copyright (C) 2006-2011 Christoph Sommer <christoph.sommer@uibk.ac.at>
//
// Documentation for these modules is at http://veins.car2x.org/
//
// This program is free software; you can redistribute it and/or modify
// it under the terms of the GNU General Public License as published by
// the Free Software Foundation; either version 2 of the License, or
// (at your option) any later version.
//
// This program is distributed in the hope that it will be useful,
// but WITHOUT ANY WARRANTY; without even the implied warranty of
// MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
// GNU General Public License for more details.
//
// You should have received a copy of the GNU General Public License
// along with this program; if not, write to the Free Software
// Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
//

#include "application/traci/TraCIDemo11p.h"

using Veins::TraCIMobilityAccess;
using Veins::AnnotationManagerAccess;

const simsignalwrap_t TraCIDemo11p::parkingStateChangedSignal = simsignalwrap_t(TRACI_SIGNAL_PARKING_CHANGE_NAME);

Define_Module(TraCIDemo11p);

void TraCIDemo11p::initialize(int stage) {
	BaseWaveApplLayer::initialize(stage);
	if (stage == 0) {
		traci = TraCIMobilityAccess().get(getParentModule());
		annotations = AnnotationManagerAccess().getIfExists();
		ASSERT(annotations);
		this->denmRepition = 0;
		this->denmInterval = 5.0;
		this->trigged = false;
		this->hopCount = 10;
		this->hasRebroadcasted = false;
		this->counter = 0;


		//sentMessage = false;
		lastDroveAt = simTime();
		findHost()->subscribe(parkingStateChangedSignal, this);
		isParking = false;
		sendWhileParking = par("sendWhileParking").boolValue();


		//for CBF forwaring:
		this->cbfBuffer = "tom";

		//CBF slutt
	}
}

void TraCIDemo11p::onBeacon(WaveShortMessage* wsm) {

    //std::cout << wsm->getSenderAddress() << std::endl;
}

void TraCIDemo11p::onData(WaveShortMessage* wsm) {
	findHost()->getDisplayString().updateWith("r=16,green");
	annotations->scheduleErase(1, annotations->drawLine(wsm->getSenderPos(), traci->getPositionAt(simTime()), "blue"));


	//Uten event-gjenkjenning - 25%
    this->hostId = wsm->getWsmData();
    bool found = false;
    int index = atoi(this->hostId.c_str());
    std::string test = this->nodeArray[index];
    for (int i = 0; i < 1500;i++){
        if (this->nodeArray[i] == this->hostId){
            found = true;
            break;
        }
    }
    if (found){

    }
    else {
        this->rebroadcast = (rand() % 100) > 25;
        this->nodeArray[counter] = this->hostId;
        counter++;
        if (this->rebroadcast){
           sendMessage(this->hostId);
        }
    }

}

void TraCIDemo11p::sendMessage(std::string blockedRoadId) {
    this->denmRepition--;
    this->lastSent = simTime();
	t_channel channel = dataOnSch ? type_SCH : type_CCH;
	WaveShortMessage* wsm = prepareWSM("data", dataLengthBits, channel, dataPriority, -1,2);
	wsm->setWsmData(blockedRoadId.c_str());
	//this->cbfBuffer = wsm->getWsmData();
	sendWSM(wsm);
}
void TraCIDemo11p::receiveSignal(cComponent* source, simsignal_t signalID, cObject* obj) {
	Enter_Method_Silent();
	if (signalID == mobilityStateChangedSignal) {
		handlePositionUpdate(obj);
	}
	else if (signalID == parkingStateChangedSignal) {
		handleParkingUpdate(obj);
	}
}
void TraCIDemo11p::handleParkingUpdate(cObject* obj) {
	isParking = traci->getParkingState();
	if (sendWhileParking == false) {
		if (isParking == true) {
			(FindModule<BaseConnectionManager*>::findGlobalModule())->unregisterNic(this->getParentModule()->getSubmodule("nic"));
		}
		else {
			Coord pos = traci->getCurrentPosition();
			(FindModule<BaseConnectionManager*>::findGlobalModule())->registerNic(this->getParentModule()->getSubmodule("nic"), (ChannelAccess*) this->getParentModule()->getSubmodule("nic")->getSubmodule("phy80211p"), &pos);
		}
	}
}
void TraCIDemo11p::handlePositionUpdate(cObject* obj) {
	BaseWaveApplLayer::handlePositionUpdate(obj);
	if (traci->getSpeed() < 1) {
		if (simTime() - lastDroveAt >= 10) {
			findHost()->getDisplayString().updateWith("r=16,red");
			int a = findHost()->getId();
			std::stringstream ss;
			ss << a;
			std::string id = ss.str();
            if (!this->trigged){
                    sendMessage(id);
                    this->trigged = true;
                    std::cout << "Oppdager og sender ut DENM at " << simTime() << std::endl;
            }

		}
	}
	else {
		lastDroveAt = simTime();
	}
}
void TraCIDemo11p::sendWSM(WaveShortMessage* wsm) {
	//if (isParking && !sendWhileParking) return;
	sendDelayedDown(wsm,individualOffset);
}
